#!/bin/sh
#
# reduce_deps() tests
#
# Copyright (C) 2014  Patrick "P. J." McDermott
#
# This file is part of the ProteanOS Development Kit.
#
# The ProteanOS Development Kit is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# The ProteanOS Development Kit is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the ProteanOS Development Kit.  If not, see
# <http://www.gnu.org/licenses/>.

. "${srcdir}/tests/common.sh"

use deps

do_test()
{
	local deps="${1}"
	local union="${2}"
	local host_arch="${3}"
	local host_plat="${4}"
	local result="${5}"

	deps="$(reduce_deps "${deps}" "${union}" "${host_arch}" "${host_plat}")"

	if [ "x${deps}" != "x${result}" ]; then
		${union} && printf 'Union' || printf 'Normal'
		printf ' dependency list for '
		printf 'host arch "%s" and host plat "%s":\n' \
			"${host_arch}" "${host_plat}"
		printf '  "%s" != "%s"\n' \
			"${deps}" "${result}"
		tests_es=1
	fi
}

# Normal and union AND-lists.

do_test 'foo, bar' false 'amd64-linux-glibc' 'dev' 'foo, bar'

do_test 'foo,bar' false 'amd64-linux-glibc' 'dev' 'foo, bar'

do_test 'foo,   bar' false 'amd64-linux-glibc' 'dev' 'foo, bar'

do_test 'foo, bar' true 'amd64-linux-glibc' 'dev' 'foo, bar'

do_test 'foo,bar' true 'amd64-linux-glibc' 'dev' 'foo, bar'

do_test 'foo,   bar' true 'amd64-linux-glibc' 'dev' 'foo, bar'

# Normal and union OR-lists.

do_test 'foo | bar' false 'amd64-linux-glibc' 'dev' 'foo | bar'

do_test 'foo|bar' false 'amd64-linux-glibc' 'dev' 'foo | bar'

do_test 'foo   |   bar' false 'amd64-linux-glibc' 'dev' 'foo | bar'

# Normal and union AND-lists with arch specs.

do_test 'foo [amd64-linux-glibc], bar [!amd64-linux-glibc]' false \
	'amd64-linux-glibc' 'dev' 'foo'

do_test 'foo [amd64-linux-glibc], bar [!amd64-linux-glibc]' true \
	'amd64-linux-glibc' 'dev' 'foo'

# Normal and union AND-lists with plat specs.

do_test 'foo <dev>, bar <!dev>' false 'amd64-linux-glibc' 'dev' 'foo'

do_test 'foo <dev>, bar <!dev>' true 'amd64-linux-glibc' 'dev' 'foo'
